part of '../flutterx_live_data.dart';

/// A [LiveData] instance which can never changes and cannot have active observers. The value is immediately dispatched
/// Use this class only as fallback when other LiveData are not provided
/// E.g. LiveDataBuilder<String>(data: myData ?? const ImmutableLiveData('no value'), builder: ...);
class ImmutableLiveData<T> implements LiveData<T> {
  @override
  Map<Observer<T>, ObserverWrapper<T>> get _observers => throw UnimplementedError();

  @override
  int get _activeCount => 0;

  @override
  set _activeCount(int value) => throw UnsupportedError('cannot set unmodifiable data');

  @override
  final T _data;

  @override
  set _data(T value) => throw UnsupportedError('cannot set unmodifiable data');

  @override
  int get _version => 0;

  @override
  set _version(int value) => throw UnsupportedError('cannot set unmodifiable data');

  @override
  T get value => _data;

  @override
  int get version => 0;

  /// Creates with value
  const ImmutableLiveData(this._data);

  @override
  bool get initialized => true;

  @override
  bool get isActive => false;

  @override
  void onActive() {}

  @override
  void onInactive() {}

  @override
  bool shouldUpdateValue(T oldValue, T newValue) => false;

  @override
  void postData(FutureOr<T> value) => throw UnsupportedError('cannot set unmodifiable data');

  @override
  void setData(T value) => throw UnsupportedError('cannot set unmodifiable data');

  @override
  void invalidate() {}

  @override
  ObserverWrapper<T> observeForever(Observer<T> onChanged, {Dispatcher? dispatcher}) =>
      addObserverWrapper(ObserverWrapper<T>(this, onChanged, dispatcher));

  @override
  void removeObserver(Observer<T> observer) {}

  @override
  ObserverWrapper<T> addObserverWrapper(ObserverWrapper<T> observer) => observer.._considerNotify();

  @override
  void removeObserverWrapper(ObserverWrapper<T> observer) {}

  @override
  void _changeActiveState(ObserverWrapper<T> observer, {required bool active}) {}

  @override
  void _dispatchingValue(ObserverWrapper<T>? initiator) {}

  @override
  String toString() => '${objectRuntimeType(this, 'LiveData')}($value)';
}
