part of '../flutterx_live_data.dart';

/// A [LiveData] instance which dispatch it's value only when [send] is called
/// Contrary to LiveData, you can call [send] with the same value multiple times, it will always be notified
/// By default a LiveData dispatch it's current data to newly activated observers, LiveEvent instead notifies
/// it's observers only once when you call [send].
/// If an [Observer] it's added after an event is sent it won't be notified of that event, anyway it can be
/// notified of the following events sent as long as it remains active.
class LiveEvent<T> extends LiveData<T> {
  @override
  bool shouldUpdateValue(T oldValue, T newValue) => true;

  @override
  @Deprecated('Use postSend')
  void postData(FutureOr<T> value) => super.postData(value);

  @override
  @Deprecated('Use send')
  void setData(T value) => send(value);

  @override
  void invalidate() {}

  /// Calls [send] after future task completion
  void postSend(FutureOr<T> value) => super.postData(value);

  /// Send event to active observers
  void send(T value) {
    _data = value;
    _version++;
    super._dispatchingValue(null);
  }

  @override
  void _dispatchingValue(ObserverWrapper<T>? initiator) => initiator == null ? super._dispatchingValue(null) : null;
}

/// [MediatorMixin] implementation of [LiveEvent]
class MediatorLiveEvent<T> extends LiveEvent<T> with MediatorMixin<T> {
  @override
  bool get reactive => false;
}
