import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:flutterx_live_data/flutterx_live_data.dart';

void main() {
  group('Test LiveData', () {
    testWidgets('Test Map', testLiveDataMap);
  });
}

Future<void> testLiveDataMap(WidgetTester tester) async {
  final src = MutableLiveData(initialValue: 0);
  final data = MediatorLiveData<String>()..addTransformSource(src, (value) => 'number $value');
  // Src value is initialized
  expect(src.value, 0);
  // Updating source value
  src.value++;
  await tester.pumpWidget(MaterialApp(builder: (context, _) {
    // Changing value during build
    src.value = 666;
    expect(src.value, 666);
    return LiveDataBuilder(data: data, builder: (context, value) => Text(value.toString()));
  }));
  // After build value has been dispatched
  expect(data.value, 'number 666');
  src.value = 777;
  // Value has been updated since computed value has active observers now
  expect(data.value, 'number 777');
  await tester.pumpFrames(
      MaterialApp(
          builder: (context, _) => CombinedLiveDataBuilder.with2<int, int, String>(
              x1: src,
              x2: src.liveMap((p0) => p0 * 2),
              transform: (a, b) => 'result: $a $b',
              builder: (context, value) => Text(value))),
      const Duration(seconds: 1));
  expect(find.text('result: 777 1554'), findsOneWidget);
}
